#ifndef _KUERZWEGE_H_
#define _KUERZWEGE_H_

// Hilfsprogramme 
#include <iostream>
#include "math.h"
using namespace std;
#include <cstdlib>

void ChooseNachbar(int* R,int v,int* column_indices,
           double* entries,int* row_offsets,int *w)
{
  // Bestimmt Nachbar w von v in der Restmenge R
  // w=-1 falls nicht da!
  // Input: Graph im CSR-Format: column_indiced,entries,row_offets
  //        Knoten v:
  // InOut: Restmenge R: gespeichert als 0-1-Feld (nur 1)
  //        
  // Output: w Nachbar in R 

  int k1,k2,k,i;
  double wij;
  k1=row_offsets[v];
  k2=row_offsets[v+1];
  k=k1;
  *w=-1;
  while (k<k2)
    {
    i=column_indices[k];
    wij=entries[k];
    if ((wij!=0)&&(R[i]!=1))
      {
        *w=i;
        R[i]=1;
	k=k2;
      }
    k++;
    } 
}

void BFS(int n,int s,int* column_indices,double* entries,
                                int* row_offsets,int* p)
// Bestimmt in eine ger. Graphen mit BFS 
// den kuerzesten Weg (ohne Gewicht)

// Achtung: Der Graph kann Kanten mit Gewicht 0 enthalten, 
// diese sind nicht vorhanden
// Optimal fuer Graphen mit wenigen Kanten

// R zeigt Die Menge der Knoten, 
//     die bereits behandelt wurden an (0 oder 1).
// Q die Menge der Knoten in der Warteschlange 
// (von Qmin bis Qmax)

// Input: n Anzahl Knoten
//        s Startknoten
//        Graph im CSR-Format: column_indiced,entries,row_offets
// Output: Baum Speicherung in p (jeweils Vorgaenger gespeichert!!!)
//  bei p(v)=-1 kein Vorgaenger gefunden   


{ 
  int* R;
  int* Q;
  int k1,k2,i,k,v,w,Qmin,Qmax;
  Q=(int*)calloc(n,sizeof(int)); // Speicher allokieren
  R=(int*)calloc(n,sizeof(int));
  vicopy(n,Q,0); // Q=0
  vicopy(n,R,0); // R=0
  vicopy(n,p,-1); // p=(-1,-1,..-1)
  R[s]=1;
  Q[0]=s;
  Qmin=0;
  Qmax=1;
  while (Qmin!=Qmax)
    {
     v=Q[Qmin]; 
     // Bestimmung der Nachbarn
     ChooseNachbar(R,v,column_indices,entries,row_offsets,&w); 
     if (w<0)
       {
	 Qmin++;
       } 
     else
       {
         Q[Qmax]=w;
         Qmax++;
         p[w]=v;
       }   
    } 
  free(Q); // Speicher freigeben
  free(R);
}

void suche_lmin(double* l,int* Q,int *j,int n)
{
  int i;
  double lmin;
  lmin=pow(10,40);
  for (i=0;i<n;i++)
    {
       if (Q[i]!=1) 
      
       {
      	  if (l[i]<lmin)
            {
      	    *j=i;
      	    lmin=l[i]; 
            }
        } 
     }
}

void dijkstra(int n,int s,int* column_indices,double* entries,
                            int* row_offsets,int* p,double* l)
// Bestimmt kuerzeste Wege nach Dijkstra
// Input: n Anzahl Knoten
//        s Startknoten
//        gew. Graph im CSR-Format: column_indiced,entries,row_offets
//        dabei Gewichte in entries gespeichert.
// Output: Baum Speicherung in p (jeweils Vorgaenger gespeichert!!!)
//  bei p(v)=-1 kein Vorgaenger gefunden  
//         l(v).. Laenge des kuerzesten Weges

{ 
  const double oxer=pow(10,40);  
  int* Q;
  int k1,k2,i,k,j,cardQ;
  double wij;
  Q=(int*)calloc(n,sizeof(int));
  // Q[j]=1 zeigt an, dass diese bereits als kuerzesten Weg gefunden
  // cardQ misst die Anzahl Einsen in Q
  vdcopy(n,l,oxer);
  // oxer ist eine sehr grosse Zahl, entspr. unendlich
  vicopy(n,Q,0);
  l[s]=0;
  cardQ=0;
  while (cardQ!=n) 
    {
    suche_lmin(l,Q,&j,n); //Suche Wert mit Minimum
    Q[j]=1;
    cardQ++;
    // Update aller Nachbarn
    k1=row_offsets[j];
    k2=row_offsets[j+1]; 
    for (k=k1;k<k2;k++)
      {
	i=column_indices[k];
        wij=entries[k];
        if (l[i]>(l[j]+wij))
	  {
	    l[i]=l[j]+wij;
            p[i]=j;
          }
      }
    }
  free(Q);
}

void MooreBellmannFord(int n,int s,int* column_indices,
            double* entries,int* row_offsets,int* p,double* l)
// Bestimmt kuerzeste Wege nach Dijkstra
// Input: n Anzahl Knoten
//        s Startknoten
//        gew. Graph im CSR-Format: column_indiced,entries,row_offets
//        dabei Gewichte in entries gespeichert.
// Output: Baum Speicherung in p (jeweils Vorgaenger gespeichert!!!)
//  bei p(v)=-1 kein Vorgaenger gefunden  
//         l(v).. Laenge des kuerzesten Weges


{ 
  const double oxer=pow(10,40);
 
  int m,mi,h,k1,k2,i,k,j,cardQ;
  double wij;
  vdcopy(n,l,oxer);
  l[s]=0;
  m=row_offsets[n]; //anzahl an Kanten
  for (h=1;h<n;h++)
    {
      j=0;
      for (mi=0;mi<m;mi++) //gehe ueber alle Kanten (j,i)
	// j und i muessen mit gewicht ausgelesen werden.
        {
          i=column_indices[mi];  
          wij=entries[mi];
          if (mi>=row_offsets[j+1]) // neue Zeile in der Matrix
	  {
            j++; 
          }
          if (l[i]>(l[j]+wij))
	  {
	    l[i]=l[j]+wij;
            p[i]=j;
          }
        }
    }
} 
 
#endif
